@@ -1,3 +1,4 @@
+# pylint:disable=import-outside-toplevel,unused-import,no-member
 import asyncio
 import sys
 import os
@@ -107,7 +108,7 @@ def macos_bigsur_wants_layer():
         os.environ['QT_MAC_WANTS_LAYER'] = '1'
 
 
-def start_management(filepath=None, use_daemon=False):
+def start_management(filepath=None, use_daemon=None):
 
     if sys.platform == "darwin":
         macos_bigsur_wants_layer()
@@ -120,11 +121,26 @@ def start_management(filepath=None, use_daemon=False):
 
     from PySide2.QtWidgets import QApplication, QSplashScreen, QMessageBox
     from PySide2.QtGui import QFontDatabase, QPixmap, QIcon
-    from PySide2.QtCore import Qt
+    from PySide2.QtCore import Qt, QCoreApplication
 
     from .config import FONT_LOCATION, IMG_LOCATION, Conf
     from .ui.css import refresh_theme
 
+    # Enable High-DPI support
+    # https://stackoverflow.com/questions/35714837/how-to-get-sharp-ui-on-high-dpi-with-qt-5-6
+    if ("QT_DEVICE_PIXEL_RATIO" not in os.environ
+        and "QT_AUTO_SCREEN_SCALE_FACTOR" not in os.environ
+        and "QT_SCALE_FACTOR" not in os.environ
+        and "QT_SCREEN_SCALE_FACTORS" not in os.environ
+        ):
+        QCoreApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
+    # No more rounding
+    # https://github.com/pyqtgraph/pyqtgraph/issues/756
+    # https://lists.qt-project.org/pipermail/development/2019-September/037434.html
+    QApplication.setHighDpiScaleFactorRoundingPolicy(Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)
+    # Use highDPI pixmaps
+    QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)
+
     app = QApplication(sys.argv)
     app.setApplicationDisplayName("angr management")
     app.setApplicationName("angr management")
@@ -136,26 +152,6 @@ def start_management(filepath=None, use_daemon=False):
 
     refresh_theme()
 
-    # URL scheme
-    from .logic.url_scheme import AngrUrlScheme
-    scheme = AngrUrlScheme()
-    registered, _ = scheme.is_url_scheme_registered()
-    supported = scheme.is_url_scheme_supported()
-    if not registered and supported:
-        btn = QMessageBox.question(None, "Setting up angr URL scheme",
-                "angr URL scheme allows \"deep linking\" from browsers and other applications by registering the "
-                "angr:// protocol to the current user. Do you want to register it? You may unregister at any "
-                "time in Preferences.",
-                defaultButton=QMessageBox.Yes)
-        if btn == QMessageBox.Yes:
-            try:
-                AngrUrlScheme().register_url_scheme()
-            except (ValueError, FileNotFoundError) as ex:
-                QMessageBox.warning(None, "Error in registering angr URL scheme",
-                        "Failed to register the angr URL scheme.\n"
-                        "The following exception occurred:\n"
-                        + str(ex))
-
     # Make + display splash screen
     splashscreen_location = os.path.join(IMG_LOCATION, 'angr-splash.png')
     splash_pixmap = QPixmap(splashscreen_location)
@@ -171,8 +167,6 @@ def start_management(filepath=None, use_daemon=False):
 
     from .logic import GlobalInfo
     from .ui.main_window import MainWindow
-    from .daemon import daemon_exists, run_daemon_process, daemon_conn
-    from .daemon.client import ClientService
 
     # Load fonts
     QFontDatabase.addApplicationFont(os.path.join(FONT_LOCATION, "SourceCodePro-Regular.ttf"))
@@ -185,30 +179,8 @@ def start_management(filepath=None, use_daemon=False):
 
     GlobalInfo.gui_thread = threading.get_ident()
 
-    if use_daemon:
-        # connect to daemon (if there is one)
-        if not daemon_exists():
-            print("[+] Starting a new daemon.")
-            run_daemon_process()
-            time.sleep(0.2)
-        else:
-            print("[+] Connecting to an existing angr management daemon.")
-
-        while True:
-            try:
-                GlobalInfo.daemon_conn = daemon_conn(service=ClientService)
-            except ConnectionRefusedError:
-                print("[-] Connection failed... try again.")
-                time.sleep(0.4)
-                continue
-            print("[+] Connected to daemon.")
-            break
-
-        from rpyc import BgServingThread
-        th = BgServingThread(GlobalInfo.daemon_conn)
-
     file_to_open = filepath if filepath else None
-    main_window = MainWindow()
+    main_window = MainWindow(app=app, use_daemon=use_daemon)
     splash.finish(main_window)
 
     if file_to_open is not None:
@@ -230,7 +202,7 @@ def main():
                                                                          "automatically start a daemon if there isn't "
                                                                          "already one running.")
     parser.add_argument("-D", "--daemon", action='store_true', help="start a daemon to handle angr:// URLs.")
-    parser.add_argument("-u", "--url", type=str, help="(internal) handle angr:// URLs. the daemon must be running.")
+    parser.add_argument("-u", "--url", type=str, nargs='?', help="(internal) handle angr:// URLs. the daemon must be running.")
     parser.add_argument("binary", nargs="?", help="the binary to open (for the GUI)")
 
     args = parser.parse_args()
@@ -261,7 +233,7 @@ def main():
         import IPython
         IPython.embed(banner1="")
     if not args.no_gui:
-        start_management(args.binary, use_daemon=args.with_daemon)
+        start_management(args.binary, use_daemon=True if args.with_daemon else None)
 
 if __name__ == '__main__':
     main()
