@@ -52,9 +52,28 @@ def font_serializer(config_option, value: QFont) -> str:
     return f'{value.pointSize()}px {value.family()}'
 
 
+def bool_parser(config_option, value) -> bool:
+    if not value:
+        return False
+    if value.lower() in {"true", "1", "yes"}:
+        return True
+    if value.lower() in {"false", "0", "no"}:
+        return False
+    _l.error("Failed to parse value %r as bool for option %s. Default to False.", value, config_option)
+    return False
+
+
+def bool_serializer(config_option, value: bool) -> str:
+    if not isinstance(value, bool):
+        _l.error("Failed to serialize value %r as bool for option %s. Default to False.", value, config_option)
+        return "false"
+    return "true" if value else "false"
+
+
 data_serializers = {
     QColor: (color_parser, color_serializer),
-    QFont: (font_parser, font_serializer)
+    QFont: (font_parser, font_serializer),
+    bool: (bool_parser, bool_serializer),
 }
 
 
@@ -67,12 +86,14 @@ ENTRIES = [
     CE('code_font', QFont, QFont("Source Code Pro", 10)),
 
     CE('theme_name', str, "Light"),
+    CE('disasm_view_minimap_viewport_color', QColor, QColor(0xFF, 0x00, 0x00)),
     CE('disasm_view_operand_color', QColor, QColor(0x00, 0x00, 0x80)),
     CE('disasm_view_operand_constant_color', QColor, QColor(0x00, 0x00, 0x80)),
     CE('disasm_view_variable_label_color', QColor, QColor(0x00, 0x80, 0x00)),
     CE('disasm_view_operand_highlight_color', QColor, QColor(0xfc, 0xef, 0x00)),
     CE('disasm_view_operand_select_color', QColor, QColor(0xff, 0xff, 0x00)),
     CE('disasm_view_function_color', QColor, QColor(0x00, 0x00, 0xff)),
+    CE('disasm_view_ir_default_color', QColor, QColor(0x80, 0x80, 0x80)),
     CE('disasm_view_label_color', QColor, QColor(0x00, 0x00, 0xff)),
     CE('disasm_view_label_highlight_color', QColor, QColor(0xf0, 0xf0, 0xbf)),
     CE('disasm_view_target_addr_color', QColor, QColor(0x00, 0x00, 0xff)),
@@ -97,6 +118,7 @@ ENTRIES = [
     CE('function_table_plt_color', QColor, QColor(0x00, 0x80, 0x00)),
     CE('function_table_simprocedure_color', QColor, QColor(0x80, 0x00, 0x00)),
     CE('function_table_alignment_color', QColor, QColor(0x80, 0x00, 0x80)),
+    CE('function_table_signature_bg_color', QColor, QColor(0xaa, 0xff, 0xff)),
     CE('palette_window', QColor, QColor(0xef, 0xef, 0xef, 0xff)),
     CE('palette_windowtext', QColor, QColor(0x00, 0x00, 0x00, 0xff)),
     CE('palette_base', QColor, QColor(0xff, 0xff, 0xff, 0xff)),
@@ -119,6 +141,19 @@ ENTRIES = [
     CE('palette_shadow', QColor, QColor(0x76, 0x76, 0x76, 0xff)),
     CE('palette_link', QColor, QColor(0x00, 0x00, 0xff, 0xff)),
     CE('palette_linkvisited', QColor, QColor(0xff, 0x00, 0xff, 0xff)),
+    CE('pseudocode_comment_color', QColor, QColor(0x00, 0x80, 0x00, 0xff)),
+    CE('pseudocode_function_color', QColor, QColor(0x00, 0x00, 0xff, 0xff)),
+    CE('pseudocode_quotation_color', QColor, QColor(0x00, 0x80, 0x00, 0xff)),
+    CE('pseudocode_keyword_color', QColor, QColor(0x00, 0x00, 0x80, 0xff)),
+
+    # FLIRT signatures
+    CE('flirt_signatures_root', str, "./flirt_signatures/"),
+
+    # Library documentation
+    CE('library_docs_root', str, "./library_docs/"),
+
+    # last-used directory
+    CE('last_used_directory', str, ''),
 
     # feature map
     CE('feature_map_color_regular_function', QColor, QColor(0x00, 0xa0, 0xe8)),
@@ -132,12 +167,18 @@ ENTRIES = [
 
     # plugins
     CE('plugin_search_path', str, '$AM_BUILTIN_PLUGINS:~/.local/share/angr-management/plugins'),
-    CE('plugin_blacklist', str, 'sample_plugin,log_fatigue_plugin'),
-    CE('enabled_plugins', str, 'binsync,trace_viewer,dep_viewer,'),
+    CE('plugin_blacklist', str, 'sample_plugin,log_fatigue_plugin,log_reverse_engineering_plugin'),
+    CE('enabled_plugins', str, 'binsync,trace_viewer,dep_viewer'),
 
     # configurations for individual plugins
     # TOOD: Move them to separate locations
     CE('plugin_ComponentsPlugin_enabled', bool, False),
+
+    # VARec
+    CE('varec_endpoint', str, "http://192.168.32.129:5000/varec"),
+
+    # Daemon
+    CE('use_daemon', bool, False),
 ]
 
 
@@ -156,12 +197,24 @@ class ConfigurationManager:
 
         if entries is None:
             self._entries = { }
-
-            for entry in ENTRIES:
-                self._entries[entry.name] = entry.copy()
+            self.load_initial_entries(reset=True)
         else:
             self._entries = entries
 
+    def load_initial_entries(self, reset: bool=True):
+        """
+        Load configuration entries into self._entries.
+
+        :param reset:   Reset all configuration items to their default values.
+        :return:        None
+        """
+        for entry in ENTRIES:
+            if entry.name not in self._entries:
+                self._entries[entry.name] = entry.copy()
+            else:
+                if reset:
+                    self._entries[entry.name] = entry.copy()
+
     @staticmethod
     def _manage_font_cache(real_font, font, metrics, height, width, ascent):
         if real_font == font:
@@ -212,6 +265,10 @@ class ConfigurationManager:
             self._code_font_width,
             self._code_font_ascent)
 
+    disasm_font: QFont
+    symexec_font: QFont
+    code_font: QFont
+
     @property
     def disasm_font_metrics(self):
         self._disasm_manage_font_cache()
@@ -304,7 +361,7 @@ class ConfigurationManager:
         return list(super().__dir__()) + list(self._entries)
 
     @classmethod
-    def parse(cls, f):
+    def parse(cls, f, ignore_unknown_entries: bool=True):
         entry_map = {}
         for entry in ENTRIES:
             entry_map[entry.name] = entry.copy()
@@ -314,8 +371,14 @@ class ConfigurationManager:
 
             for k, v in loaded.items():
                 if k not in entry_map:
-                    _l.warning('Unknown configuration option \'%s\'. Ignoring...', k)
-                    continue
+                    if ignore_unknown_entries:
+                        _l.warning('Unknown configuration option \'%s\'. Ignoring...', k)
+                        continue
+                    else:
+                        # default to a string
+                        entry = CE(k, str, v)
+                        entry_map[k] = entry
+
                 entry = entry_map[k]
 
                 if entry.type_ in data_serializers:
@@ -327,15 +390,15 @@ class ConfigurationManager:
                              v, k, type(v), entry.type_)
                     continue
                 entry.value = v
-        except toml.TomlDecodeError as e:
-            _l.error('Failed to parse configuration file: \'%s\'. Continuing with default options...', e.msg)
+        except toml.TomlDecodeError as ex:
+            _l.error('Failed to parse configuration file: \'%s\'. Continuing with default options...', ex.msg)
 
         return cls(entry_map)
 
     @classmethod
-    def parse_file(cls, path:str):
+    def parse_file(cls, path: str, ignore_unknown_entries: bool=True):
         with open(path, 'r') as f:
-            return cls.parse(f)
+            return cls.parse(f, ignore_unknown_entries=ignore_unknown_entries)
 
     def save(self, f):
         out = {}
@@ -367,7 +430,7 @@ class ConfigurationManager:
             if os.path.isfile(initial_config_path):
                 from . import save_config  # delayed import
                 # we found it!
-                new_conf = self.__class__.parse_file(initial_config_path)
+                new_conf = self.__class__.parse_file(initial_config_path, ignore_unknown_entries=False)
                 # copy entries over
                 self._entries = new_conf._entries
                 # save it!
