@@ -5,25 +5,28 @@ from typing import List, Optional, Type, Union, Callable, TYPE_CHECKING
 
 import angr
 from angr.block import Block
+from angr.knowledge_base import KnowledgeBase
 from angr.analyses.disassembly import Instruction
 
 from .jobs import CFGGenerationJob
 from .object_container import ObjectContainer
-from .sync_ctrl import SyncControl
 from ..logic import GlobalInfo
 from ..logic.threads import gui_thread_schedule_async
-from ..daemon.client import DaemonClient
 
 if TYPE_CHECKING:
     from ..ui.workspace import Workspace
 
 
 class Instance:
+    """
+    An object to give access to normal angr project objects like a Project, CFG, and other analyses.
+    """
     project: Union[angr.Project, ObjectContainer]
     cfg: Union[angr.analyses.cfg.CFGBase, ObjectContainer]
     cfb: Union[angr.analyses.cfg.CFBlanket, ObjectContainer]
 
     def __init__(self):
+        # pylint:disable=import-outside-toplevel)
         # delayed import
         from ..ui.views.interaction_view import PlainTextProtocol, ProtocolInteractor, SavedInteraction
 
@@ -32,6 +35,7 @@ class Instance:
 
         self.jobs = []
         self._jobs_queue = Queue()
+        self.current_job = None
 
         self.extra_containers = {}
         self._container_defaults = {}
@@ -49,7 +53,8 @@ class Instance:
         self.register_container('cfb', lambda: None, Optional[angr.analyses.cfg.CFBlanket], "The current CFBlanket")
         self.register_container('interactions', lambda: [], List[SavedInteraction], 'Saved program interactions')
         # TODO: the current setup will erase all loaded protocols on a new project load! do we want that?
-        self.register_container('interaction_protocols', lambda: [PlainTextProtocol], List[Type[ProtocolInteractor]], 'Available interaction protocols')
+        self.register_container('interaction_protocols', lambda: [PlainTextProtocol], List[Type[ProtocolInteractor]],
+                                'Available interaction protocols')
 
         self.project.am_subscribe(self.initialize)
 
@@ -58,9 +63,10 @@ class Instance:
         self._label_rename_callback = None  # type: Union[None, Callable[[int, str], None]]      #  (addr, new_name)
         self._set_comment_callback = None  # type: Union[None, Callable[[int, str], None]]       #  (addr, comment_text)
 
-        self.sync = SyncControl(self)
         self.cfg_args = None
+        self.variable_recovery_args = None
         self._disassembly = {}
+        self.pseudocode_variable_kb = None
 
         self._start_worker()
 
@@ -137,25 +143,32 @@ class Instance:
             self._container_defaults[name] = (default_val_func, ty)
             self.extra_containers[name] = ObjectContainer(default_val_func(), description)
 
-    def initialize(self, initialized=False, cfg_args=None, **kwargs):  # pylint: disable=unused-argument
+    def initialize(self, initialized=False, cfg_args=None, variable_recovery_args=None, **kwargs):  # pylint:disable=unused-argument
         if self.project.am_none:
             return
 
-        DaemonClient.register_binary(self.project.loader.main_object.binary,
-                                     self.project.loader.main_object.md5,
-                                     self.project.loader.main_object.sha256)
-
         if not initialized:
+            if self.pseudocode_variable_kb is None:
+                self.initialize_pseudocode_variable_kb()
+
             if cfg_args is None:
                 cfg_args = {}
             # save cfg_args
             self.cfg_args = cfg_args
 
+            if variable_recovery_args is None:
+                variable_recovery_args = {}
+            self.variable_recovery_args = variable_recovery_args
+
             # generate CFG
             cfg_job = self.generate_cfg()
 
             # start daemon
             self._start_daemon_thread(self._refresh_cfg, 'Progressively Refreshing CFG', args=(cfg_job,))
+        self.workspace.plugins.handle_project_initialization()
+
+    def initialize_pseudocode_variable_kb(self):
+        self.pseudocode_variable_kb = KnowledgeBase(self.project.am_obj, name="pseudocode_variable_kb")
 
     def generate_cfg(self):
         cfg_job = CFGGenerationJob(
@@ -194,11 +207,26 @@ class Instance:
                     return insn_piece.render()[0]
         return None
 
+    def interrupt_current_job(self):
+        """Notify the current running job that the user requested an interrupt. The job may ignore it."""
+        # Due to thread scheduling, current_job reference *must* first be saved on the stack. Accessing self.current_job
+        # multiple times will lead to a race condition.
+        current_job = self.current_job
+        if current_job:
+            current_job.keyboard_interrupt()
+
     def join_all_jobs(self):
         # ...lol
         while self.jobs:
             time.sleep(0.05)
 
+    def append_code_to_console(self, hook_code_string):
+        console = self.workspace._get_or_create_console_view()
+        console.set_input_buffer(hook_code_string)
+
+    def delete_hook(self, addr):
+        self.project.unhook(addr)
+
     #
     # Private methods
     #
@@ -221,13 +249,17 @@ class Instance:
             gui_thread_schedule_async(self._set_status, args=("Working...",))
 
             try:
+                self.current_job = job
                 result = job.run(self)
-            except Exception as e:  # pylint: disable=broad-except
+                self.current_job = None
+            except Exception as e: # pylint: disable=broad-except
+                self.current_job = None
                 self.workspace.log('Exception while running job "%s":' % job.name)
                 self.workspace.log(e)
             else:
                 gui_thread_schedule_async(job.finish, args=(self, result))
 
+    # pylint:disable=no-self-use
     def _set_status(self, status_text):
         GlobalInfo.main_window.status = status_text
 
@@ -246,6 +278,7 @@ class Instance:
                 break
 
     def _reset_containers(self, **kwargs):
+        # pylint:disable=consider-using-dict-items
         for name in self.extra_containers:
             self.extra_containers[name].am_obj = self._container_defaults[name][0]()
             self.extra_containers[name].am_event(**kwargs)
