@@ -5,16 +5,19 @@ import os
 
 from PySide2.QtGui import QColor
 
-from angrmanagement.ui.menus.menu import MenuEntry, MenuSeparator
-from angrmanagement.ui.toolbars.toolbar import ToolbarAction
-from angrmanagement.daemon.url_handler import register_url_action, UrlActionBinaryAware
-from angrmanagement.daemon.client import DaemonClient
+from ..config import config_path
+from ..config.config_manager import ENTRIES
+from ..ui.menus.menu import MenuEntry, MenuSeparator
+from ..ui.toolbars.toolbar import ToolbarAction
+from ..daemon.url_handler import register_url_action, UrlActionBinaryAware
+from ..daemon.client import DaemonClient
+from ..ui.widgets.qblock import QBlock
 from ..config import Conf
 from . import load_plugins_from_dir
 from .base_plugin import BasePlugin
 
 if TYPE_CHECKING:
-    from angrmanagement.ui.workspace import Workspace
+    from ..ui.workspace import Workspace
 
 
 l = logging.getLogger(__name__)
@@ -50,7 +53,7 @@ class PluginManager:
             search_dir = os.path.expandvars(search_dir)
             for plugin_or_exception in load_plugins_from_dir(search_dir):
                 if isinstance(plugin_or_exception, Exception):
-                    l.info(plugin_or_exception)
+                    l.warning("Exception occurred during plugin loading: %s", plugin_or_exception)
                 else:
                     plugin_or_exception: Type[BasePlugin]
                     plugin_conf_key = "plugin_%s_enabled" % plugin_or_exception.__name__
@@ -90,52 +93,90 @@ class PluginManager:
         try:
             plugin = plugin_cls(self.workspace)
             self.active_plugins.append(plugin)
+            plugin.__cached_status_bar_widgets = []
             plugin.__cached_toolbar_actions = []  # a hack, lol. really this could be a mapping on PluginManager but idc
             plugin.__cached_menu_actions = []  # as above
 
             if self.workspace is not None:
-                for idx, (icon, tooltip) in enumerate(plugin_cls.TOOLBAR_BUTTONS):
-                    action = ToolbarAction(icon, 'plugin %s toolbar %d' % (plugin_cls, idx), tooltip, functools.partial(self._dispatch_single, plugin, BasePlugin.handle_click_toolbar, False, idx))
-                    plugin.__cached_toolbar_actions.append(action)
-                    self.workspace._main_window._file_toolbar.add(action)
-
-                if plugin_cls.MENU_BUTTONS:
-                    self.workspace._main_window._plugin_menu.add(MenuSeparator())
-                for idx, text in enumerate(plugin_cls.MENU_BUTTONS):
-                    action = MenuEntry(text, functools.partial(self._dispatch_single, plugin, BasePlugin.handle_click_menu, False, idx))
-                    plugin.__cached_menu_actions.append(action)
-                    self.workspace._main_window._plugin_menu.add(action)
+                self._register_status_bar_widgets(plugin)
+                self._register_toolbar_actions(plugin_cls, plugin)
+                self._register_menu_buttons(plugin_cls, plugin)
 
                 for dview in self.workspace.view_manager.views_by_category['disassembly']:
                     plugin.instrument_disassembly_view(dview)
                 for cview in self.workspace.view_manager.views_by_category['pseudocode']:
                     plugin.instrument_code_view(cview)
 
-                for action in plugin_cls.URL_ACTIONS:
-                    DaemonClient.register_handler(action,
-                                                  functools.partial(self._dispatch_single,
-                                                                    plugin,
-                                                                    BasePlugin.handle_url_action,
-                                                                    False,
-                                                                    action
-                                                                    )
-                                                  )
+                self._register_url_actions(plugin_cls, plugin)
+                self._register_configuration_entries(plugin_cls)
 
             for action in plugin_cls.URL_ACTIONS:
                 register_url_action(action, UrlActionBinaryAware)
 
-        except Exception:
+        except Exception: #pylint: disable=broad-except
             l.warning("Plugin %s failed to activate:", plugin_cls.get_display_name(),
                       exc_info=True)
         else:
             l.info("Activated plugin %s", plugin_cls.get_display_name())
 
+    def _register_status_bar_widgets(self, plugin: BasePlugin) -> None:
+        gen = plugin.status_bar_permanent_widgets()
+        if gen is not None:
+            for widget in gen:
+                self.workspace.main_window.statusBar().addPermanentWidget(widget)
+                widget.show()
+            plugin.__cached_status_bar_widgets.append(widget)
+
+    def _register_toolbar_actions(self, plugin_cls: Type[BasePlugin], plugin: BasePlugin) -> None:
+        for idx, (icon, tooltip) in enumerate(plugin_cls.TOOLBAR_BUTTONS):
+            action = ToolbarAction(icon, 'plugin %s toolbar %d' % (plugin_cls, idx), tooltip,
+                                   functools.partial(self._dispatch_single, plugin, BasePlugin.handle_click_toolbar,
+                                                     False, idx))
+            plugin.__cached_toolbar_actions.append(action)
+            self.workspace._main_window._file_toolbar.add(action)
+
+    def _register_menu_buttons(self, plugin_cls: Type[BasePlugin], plugin: BasePlugin) -> None:
+        if plugin_cls.MENU_BUTTONS:
+            self.workspace._main_window._plugin_menu.add(MenuSeparator())
+        for idx, text in enumerate(plugin_cls.MENU_BUTTONS):
+            action = MenuEntry(text,
+                               functools.partial(self._dispatch_single, plugin, BasePlugin.handle_click_menu, False,
+                                                 idx))
+            plugin.__cached_menu_actions.append(action)
+            self.workspace._main_window._plugin_menu.add(action)
+
+    def _register_url_actions(self, plugin_cls: Type[BasePlugin], plugin: BasePlugin) -> None:
+        for action in plugin_cls.URL_ACTIONS:
+            DaemonClient.register_handler(action,
+                                          functools.partial(self._dispatch_single,
+                                                            plugin,
+                                                            BasePlugin.handle_url_action,
+                                                            False,
+                                                            action
+                                                            )
+                                          )
+
+    def _register_configuration_entries(self, plugin_cls: Type[BasePlugin]) -> None:
+        new_entries_added = False
+        for ent in plugin_cls.CONFIG_ENTRIES:
+            if ent not in ENTRIES:
+                # if a plugin is disabled and then enabled, the entry may already exist?
+                ENTRIES.append(ent)
+                new_entries_added = True
+
+        if new_entries_added:
+            # reload configuration manager so that it's aware of newly added entries
+            Conf.load_initial_entries(reset=False)
+            # reload the configuration file
+            new_conf = Conf.parse_file(config_path)
+            Conf._entries = new_conf._entries
+
     def get_plugin_instance_by_name(self, plugin_cls_name: str) -> Optional[BasePlugin]:
         instances = [plugin for plugin in self.active_plugins if plugin.__class__.__name__.split(".")[-1] == plugin_cls_name]
         if not instances:
             return None
         if len(instances) > 1:
-            l.error("Somehow there is more than one instance of %s active?" % plugin_cls_name)
+            l.error("Somehow there is more than one instance of %s active?", plugin_cls_name)
         return instances[0]
 
     def get_plugin_instance(self, plugin_cls: Type[BasePlugin]) -> Optional[BasePlugin]:
@@ -143,18 +184,19 @@ class PluginManager:
         if len(instances) == 0:
             return None
         if len(instances) > 1:
-            l.error("Somehow there is more than one instance of %s active?" % plugin_cls.get_display_name())
+            l.error("Somehow there is more than one instance of %s active?", plugin_cls.get_display_name())
         return instances[0]
 
     def deactivate_plugin(self, plugin: Union[BasePlugin, Type[BasePlugin]]):
         # this method should work on both instances and classes
         if type(plugin) is type:
             plugin = self.get_plugin_instance(plugin)
-        else:
-            plugin = plugin
+
         if plugin not in self.active_plugins:
             return
 
+        for widget in plugin.__cached_status_bar_widgets:
+            self.workspace.main_window.statusBar().removeWidget(widget)
         for action in plugin.__cached_toolbar_actions:
             self.workspace._main_window._file_toolbar.remove(action)
         for action in plugin.__cached_menu_actions:
@@ -162,7 +204,8 @@ class PluginManager:
 
         try:
             plugin.teardown()
-        except Exception:
+
+        except Exception: #pylint: disable=broad-except
             l.warning("Plugin %s errored during removal. The UI may be unstable.", plugin.get_display_name(),
                       exc_info=True)
         self.active_plugins.remove(plugin)
@@ -170,25 +213,33 @@ class PluginManager:
     #
     # Dispatchers
     #
-
     def _dispatch(self, func, sensitive, *args):
         for plugin in list(self.active_plugins):
             custom = getattr(plugin, func.__name__)
             if custom.__func__ is not func:
                 try:
                     res = custom(*args)
-                except Exception as e:
+                except Exception as e: #pylint: disable=broad-except
                     self._handle_error(plugin, func, sensitive, e)
                 else:
                     yield res
 
-        return None
+    def _dispatch_with_plugin(self, func, sensitive, *args):
+        for plugin in list(self.active_plugins):
+            custom = getattr(plugin, func.__name__)
+            if custom.__func__ is not func:
+                try:
+                    res = custom(*args)
+                except Exception as e: #pylint: disable=broad-except
+                    self._handle_error(plugin, func, sensitive, e)
+                else:
+                    yield plugin, res
 
     def _dispatch_single(self, plugin, func, sensitive, *args):
         custom = getattr(plugin, func.__name__)
         try:
             return custom(*args)
-        except Exception as e:
+        except Exception as e:  #pylint: disable=broad-except
             self._handle_error(plugin, func, sensitive, e)
             return None
 
@@ -245,6 +296,14 @@ class PluginManager:
                 return True
         return False
 
+    def handle_raise_view(self, view):
+        for _ in self._dispatch(BasePlugin.handle_raise_view, False, view):
+            pass
+
+    def build_qblock_annotations(self, qblock: QBlock):
+        for res in self._dispatch(BasePlugin.build_qblock_annotations, False, qblock):
+            yield from res
+
     def build_context_menu_insn(self, insn):
         for res in self._dispatch(BasePlugin.build_context_menu_insn, False, insn):
             yield from res
@@ -257,8 +316,8 @@ class PluginManager:
         for res in self._dispatch(BasePlugin.build_context_menu_node, False, node):
             yield from res
 
-    def build_context_menu_function(self, func):
-        for res in self._dispatch(BasePlugin.build_context_menu_function, False, func):
+    def build_context_menu_functions(self, funcs):
+        for res in self._dispatch(BasePlugin.build_context_menu_functions, False, funcs):
             yield from res
 
     def get_func_column(self, idx):
@@ -279,9 +338,77 @@ class PluginManager:
             else:
                 try:
                     return plugin.extract_func_column(func, idx)
-                except Exception as e:
+                except Exception as e: #pylint: disable=broad-except
                     # this should really be a "sensitive" operation but like
                     self.workspace.log(e)
                     self.workspace.log("PLEASE FIX YOUR PLUGIN AHHHHHHHHHHHHHHHHH")
                     return 0, ''
         raise IndexError("Not enough columns")
+
+    def step_callback(self, simgr):
+        for _ in self._dispatch(BasePlugin.step_callback,True, simgr):
+            pass
+
+    def handle_variable_rename(self, func, offset: int, old_name: str, new_name: str, type_: str, size: int):
+        for res in self._dispatch(BasePlugin.handle_variable_rename, False,
+                                  func, offset, old_name, new_name, type_, size):
+            if res:
+                return True
+        return False
+
+    def handle_function_rename(self, func, old_name: str, new_name: str):
+        for res in self._dispatch(BasePlugin.handle_function_rename, False, func, old_name, new_name):
+            if res:
+                return True
+        return False
+
+    def handle_comment_changed(self, addr: int, cmt: str, new: bool, decomp: bool):
+        for res in self._dispatch(BasePlugin.handle_comment_changed, False, addr, cmt, new, decomp):
+            if res:
+                return True
+        return False
+
+    def handle_project_initialization(self):
+        for _ in self._dispatch(BasePlugin.handle_project_initialization, False):
+            pass
+
+    def handle_project_save(self, file_name: str):
+        for _ in self._dispatch(BasePlugin.handle_project_save, False, file_name):
+            pass
+
+    def on_workspace_initialized(self, workspace):
+        for _ in self._dispatch(BasePlugin.on_workspace_initialized, False, workspace):
+            pass
+
+    def angrdb_store_entries(self):
+        entries = {}
+        for plugin, res in self._dispatch_with_plugin(BasePlugin.angrdb_store_entries, False):
+            for result in res:
+                if isinstance(result, tuple) and len(result) == 2:
+                    key, value = result
+                    key = plugin.__class__.__name__.split(".")[-1] + "___" + key
+                    entries[key] = value
+        return entries
+
+    def angrdb_load_entries(self, entries):
+        plugin_name_to_plugin = {}
+        for plugin in list(self.active_plugins):
+            plugin_name_to_plugin[plugin.__class__.__name__.split(".")[-1]] = plugin
+
+        for key, value in entries.items():
+            if "___" not in key:
+                continue
+            splitted = key.split("___")
+            if len(splitted) != 2:
+                continue
+            plugin_class_name, key = splitted
+            plugin = plugin_name_to_plugin.get(plugin_class_name, None)
+            if plugin is None:
+                continue
+            # dispatch
+            custom = getattr(plugin, "angrdb_load_entry")
+            if custom is not BasePlugin.angrdb_load_entry:
+                try:
+                    custom(key, value)
+                except Exception as ex:  # pylint: disable=broad-except
+                    self._handle_error(plugin, BasePlugin.angrdb_load_entry, False, ex)
