@@ -1,20 +1,29 @@
-import sys
 import logging
 
 from PySide2.QtWidgets import QGraphicsScene, QGraphicsView, QStyleOptionGraphicsItem, QApplication,\
     QGraphicsSceneMouseEvent
 from PySide2.QtGui import QPainter, QMouseEvent, QImage, QVector2D
-from PySide2.QtCore import Qt, QSize, QEvent, QMarginsF
+from PySide2.QtCore import Qt, QSize, QEvent, QMarginsF, Signal, QRectF
 
 _l = logging.getLogger(__name__)
 
 
 class QBaseGraphicsView(QGraphicsView):
 
+    visible_scene_rect_changed = Signal(QRectF)
+
     #
     # Public methods
     #
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._visibile_scene_rect: QRectF = QRectF()
+
+    @property
+    def visible_scene_rect(self):
+        return self._visibile_scene_rect
+
     def redraw(self):
         """
         Redraw the scene. Do not recompute any items in the view.
@@ -25,22 +34,29 @@ class QBaseGraphicsView(QGraphicsView):
         if scene is not None:
             scene.update(self.sceneRect())
 
+    def viewportEvent(self, event:QEvent) -> bool:
+        visible_scene_rect = self.mapToScene(self.viewport().geometry()).boundingRect()
+        if visible_scene_rect != self._visibile_scene_rect:
+            self._visibile_scene_rect = visible_scene_rect
+            self.visible_scene_rect_changed.emit(visible_scene_rect)
 
-class QDevicePixelRatioAwareGraphicsView(QBaseGraphicsView):
+        return super().viewportEvent(event)
 
-    def currentDevicePixelRatioF(self) -> float:
-        # getting devicePixelRatio is currently broken in Qt 5.14
-        # see https://bugreports.qt.io/browse/QTBUG-53022
-        # before it is fixed, fall back to manually calculating the ratio using logicalDpiX()
-        if sys.platform == "darwin":
-            return self.logicalDpiX() / 72.0
-        elif sys.platform == "win32":
-            return self.logicalDpiX() / 96.0
-        else:
-            return self.devicePixelRatioF()
+class QSaveableGraphicsView(QBaseGraphicsView):
 
+    def __init__(self, parent=None):
+        super().__init__(parent=parent)
+        self._is_extra_render_pass: bool = False
 
-class QSaveableGraphicsView(QDevicePixelRatioAwareGraphicsView):
+    @property
+    def is_extra_render_pass(self):
+        return self._is_extra_render_pass
+
+    def set_extra_render_pass(self, is_extra_pass:bool):
+        """
+        Trigger any post-render callbacks
+        """
+        self._is_extra_render_pass = is_extra_pass
 
     def save_image_to(self, path, top_margin=50, bottom_margin=50, left_margin=50, right_margin=50):
 
@@ -71,6 +87,8 @@ class QSaveableGraphicsView(QDevicePixelRatioAwareGraphicsView):
 
 
 class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
+    ZOOM_X = True
+    ZOOM_Y = True
 
     def __init__(self, parent=None):
         super().__init__(parent=parent)
@@ -88,6 +106,9 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
 
+        # the zoom factor, for preserving the zoom
+        self.zoom_factor = None
+
         #self.setRenderHints(
                 #QPainter.Antialiasing | QPainter.SmoothPixmapTransform | QPainter.HighQualityAntialiasing)
 
@@ -97,10 +118,11 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
     def _reset_view(self):
         self.resetMatrix()
         self.centerOn(self._initial_position())
+        self.zoom(restore=True)
 
     def _reset_scene(self):
         if self.scene() is None:
-            scene = QGraphicsScene()
+            scene = QGraphicsScene(self)
             self.setScene(scene)
         else:
             self.scene().clear()
@@ -108,7 +130,7 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
     def sizeHint(self):  # pylint:disable=no-self-use
         return QSize(300, 300)
 
-    def zoom(self, out=False, at=None, reset=False):
+    def zoom(self, out=False, at=None, reset=False, restore=False):
         if at is None:
             at = self.scene().sceneRect().center().toPoint()
         lod = QStyleOptionGraphicsItem.levelOfDetailFromTransform(self.transform())
@@ -117,6 +139,8 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
 
         if reset:
             zoomFactor = 1 / lod
+        elif restore:
+            zoomFactor = self.zoom_factor if self.zoom_factor else 1 / lod
         elif not out:
             zoomFactor = zoomInFactor
         else:
@@ -129,7 +153,8 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
         oldPos = self.mapToScene(at)
 
         # Zoom
-        self.scale(zoomFactor, zoomFactor)
+        self.scale(zoomFactor if self.ZOOM_X else 1, zoomFactor if self.ZOOM_Y else 1)
+        self.zoom_factor = QStyleOptionGraphicsItem.levelOfDetailFromTransform(self.transform())
 
         # Get the new position
         newPos = self.mapToScene(at)
@@ -257,7 +282,7 @@ class QZoomableDraggableGraphicsView(QSaveableGraphicsView):
                                  event.buttons(),
                                  event.modifiers())
 
-            press_event = self.dispatchMouseEventToScene(pressy)
+            _ = self.dispatchMouseEventToScene(pressy)
 
             releasy = QMouseEvent(QEvent.MouseButtonRelease,
                                   event.pos(),
